##
## Purpose:
## This class is used by any recipes that are built using
## Cargo.

require_script cargo_common_utils.sh

# the binary we will use
: ${CARGO:="cargo"}
: ${RUSTC:="rustc"}

# In case something fails in the build process, give a bit more feedback on
# where the issue occured
export RUST_BACKTRACE="1"

# The directory of the Cargo.toml relative to the root directory, per default
# assume there's a Cargo.toml directly in the root directory
: ${CARGO_SRC_DIR:=""}
# The actual path to the Cargo.toml
: ${MANIFEST_PATH:="${S}/${CARGO_SRC_DIR}/Cargo.toml"}
: ${RUSTFLAGS:=""}

BUILD_MODE="--release"
CARGO_BUILD_FLAGS="-v --target ${HOST_SYS} ${BUILD_MODE} --manifest-path=${MANIFEST_PATH}"

# This is based on the content of CARGO_BUILD_FLAGS and generally will need to
# change if CARGO_BUILD_FLAGS changes.
BUILD_DIR="release"
CARGO_TARGET_SUBDIR="${HOST_SYS}/${BUILD_DIR}"

cargo_build () {
	export RUSTFLAGS="${RUSTFLAGS}"
	export RUST_TARGET_PATH="${RUST_TARGET_PATH}"
	log "cargo = $(which ${CARGO})"
	log "rustc = $(which ${RUSTC})"
	log "${CARGO} build ${CARGO_BUILD_FLAGS} $@"
	"${CARGO}" build ${CARGO_BUILD_FLAGS} "$@"
}

cargo_do_configure () {
	cargo_common_do_configure
}

cargo_do_compile () {
	cargo_fix_env
	cargo_build
}

cargo_do_install () {
	local have_installed=false
	for tgt in "${B}/target/${CARGO_TARGET_SUBDIR}/"*; do
		case $tgt in
		*.so|*.rlib)
			log_exec install -d "${D}${rustlibdir}"
			log_exec install -m755 "$tgt" "${D}${rustlibdir}"
			have_installed=true
			;;
		*examples)
			if [ -d "$tgt" ]; then
				for example in "$tgt/"*; do
					if [ -f "$example" ] && [ -x "$example" ]; then
						log_exec install -d "${D}${bindir}"
						log_exec install -m755 "$example" "${D}${bindir}"
						have_installed=true
					fi
				done
			fi
			;;
		*)
			if [ -f "$tgt" ] && [ -x "$tgt" ]; then
				log_exec install -d "${D}${bindir}"
				log_exec install -m755 "$tgt" "${D}${bindir}"
				have_installed=true
			fi
			;;
		esac
	done
	if ! $have_installed; then
		fatal 1 "Did not find anything to install"
	fi
}