# Common variables used by all Rust builds
export rustlibdir="${libdir}/rust"

: ${RUSTLIB:="-L ${SYSROOT_LOC}/usr/lib/rust"}
: ${RUST_DEBUG_REMAP:="--remap-path-prefix=${WORKDIR}=/usr/src/debug/${PN}/${EXTENDPE}${PV}-${PR}"}
: ${RUSTFLAGS:="${RUSTLIB} ${RUST_DEBUG_REMAP}"}
: ${RUSTLIB_DEP:="libstd-rs"}
: ${RUST_TARGET_PATH:="${NATIVE_SYSROOT_LOC}/usr/lib/rustlib"}
: ${RUST_PANIC_STRATEGY:="unwind"}



BUILD_ARCH="${recipe_build_arch}"
HOST_ARCH="${recipe_host_arch}"
TARGET_ARCH="${recipe_target_arch}"

: ${BUILD_SYS="${BUILD_ARCH}-linux"}
: ${HOST_SYS="${HOST_ARCH}-linux"}
: ${TARGET_SYS="${TARGET_ARCH}-linux"}
: ${SNAPSHOT_BUILD_SYS="${BUILD_ARCH}-unknown-linux-gnu"}

# In some cases uname and the toolchain differ on their idea of the arch name
RUST_BUILD_ARCH="${BUILD_ARCH}"

# Naming explanation
# Yocto
# - BUILD_SYS - Yocto triple of the build environment
# - HOST_SYS - What we're building for in Yocto
# - TARGET_SYS - What we're building for in Yocto
#
# So when building '-native' packages BUILD_SYS == HOST_SYS == TARGET_SYS
# When building packages for the image HOST_SYS == TARGET_SYS
# This is a gross over simplification as there are other modes but
# currently this is all that's supported.
#
# Rust
# - TARGET - the system where the binary will run
# - HOST - the system where the binary is being built
#
# Rust additionally will use two additional cases:
# - undecorated (e.g. CC) - equivalent to TARGET
# - triple suffix (e.g. CC:x86_64_unknown_linux_gnu) - both
#   see: https://github.com/alexcrichton/gcc-rs
# The way that Rust's internal triples and Yocto triples are mapped together
# its likely best to not use the triple suffix due to potential confusion.

RUST_BUILD_SYS="${BUILD_ARCH}-unknown-linux-gnu"
RUST_HOST_SYS="${HOST_ARCH}-unknown-linux-gnu"
RUST_TARGET_SYS="${TARGET_ARCH}-unknown-linux-gnu"

# wrappers to get around the fact that Rust needs a single
# binary but Yocto's compiler and linker commands have
# arguments. Technically the archiver is always one command but
# this is necessary for builds that determine the prefix and then
# use those commands based on the prefix.
: ${WRAPPER_DIR:="${WORKDIR}/wrapper"}
: ${RUST_BUILD_CC:="${WRAPPER_DIR}/build-rust-cc"}
: ${RUST_BUILD_CXX:="${WRAPPER_DIR}/build-rust-cxx"}
: ${RUST_BUILD_CCLD:="${WRAPPER_DIR}/build-rust-ccld"}
: ${RUST_BUILD_AR:="${WRAPPER_DIR}/build-rust-ar"}
: ${RUST_TARGET_CC:="${WRAPPER_DIR}/target-rust-cc"}
: ${RUST_TARGET_CXX:="${WRAPPER_DIR}/target-rust-cxx"}
: ${RUST_TARGET_CCLD:="${WRAPPER_DIR}/target-rust-ccld"}
: ${RUST_TARGET_AR:="${WRAPPER_DIR}/target-rust-ar"}

create_wrapper () {
	file="$1"
	shift

	cat <<- EOF > "${file}"
	#!/usr/bin/env python3
	import os, sys
	orig_binary = "$@"
	binary = orig_binary.split()[0]
	args = orig_binary.split() + sys.argv[1:]
	os.execvp(binary, args)
	EOF
	chmod +x "${file}"
}

export WRAPPER_TARGET_CC="${CC}"
export WRAPPER_TARGET_CXX="${CXX}"
export WRAPPER_TARGET_CCLD="${CCLD}"
export WRAPPER_TARGET_LDFLAGS="${LDFLAGS}"
export WRAPPER_TARGET_AR="${AR}"

# compiler is used by gcc-rs
# linker is used by rustc/cargo
# archiver is used by the build of libstd-rs
do_rust_create_wrappers () {
	mkdir -p "${WRAPPER_DIR}"

	# SW4Linux Build / Rust Host C compiler
	create_wrapper "${RUST_BUILD_CC}" "${BUILD_CC}"
	# SW4Linux Build / Rust Host C++ compiler
	create_wrapper "${RUST_BUILD_CXX}" "${BUILD_CXX}"
	# SW4Linux Build / Rust Host linker
	create_wrapper "${RUST_BUILD_CCLD}" "${BUILD_CCLD}" "${BUILD_LDFLAGS}"
	# SW4Linux Build / Rust Host archiver
	create_wrapper "${RUST_BUILD_AR}" "${BUILD_AR}"

	# SW4Linux Target / Rust Target C compiler
	create_wrapper "${RUST_TARGET_CC}" "${WRAPPER_TARGET_CC}" "${WRAPPER_TARGET_LDFLAGS}"
	# SW4Linux Target / Rust Target C++ compiler
	create_wrapper "${RUST_TARGET_CXX}" "${WRAPPER_TARGET_CXX}"
	# SW4Linux Target / Rust Target linker
	create_wrapper "${RUST_TARGET_CCLD}" "${WRAPPER_TARGET_CCLD}" "${WRAPPER_TARGET_LDFLAGS}"
	# SW4Linux Target / Rust Target archiver
	create_wrapper "${RUST_TARGET_AR}" "${WRAPPER_TARGET_AR}"

}

rust_common_do_configure() {
	do_rust_create_wrappers
}