#
# Utils script to generate pre-install and post-install scripts
#

generate_preinst() {
	local bin_package="$1"
	local package="$2"
	local suffix=$(shell_variable_name $package)
	local script_file="$metadataTempDir/${bin_package}.preinst"
	local templatesDir="${script_lib_dir}/xinstall"
	local templateFile="${script_lib_dir}/xinstall/template.xinst"
	local tmpDir="$metadataTempDir/.ac6_${bin_package}"
	local tmpScript="$tmpDir/tmp_${bin_package}.preinst"
	
	install -d $tmpDir
	
	# USERADD, GROUPADD, GROUPMEMS
	spec_groupadd_var=GROUPADD_PARAM_${suffix}
	spec_useradd_var=USERADD_PARAM_${suffix}
	spec_groupmems_var=GROUPMEMS_PARAM_${suffix}
	val_groupadd_set=${!spec_groupadd_var}
	val_useradd_set=${!spec_useradd_var}
	val_groupmems_set=${!spec_groupmems_var}

	if [ -n "$val_groupadd_set" -o -n "$val_useradd_set" -o -n "$val_groupmems_set" ]; then
		install ${templatesDir}/useradd.xinst $tmpDir/${bin_package}_useradd.xinst
		sed -i -e "s|@GROUPADD_PARAM@|${val_groupadd_set}|g" \
			-e "s|@USERADD_PARAM@|${val_useradd_set}|g" \
			-e "s|@GROUPMEMS_PARAM@|${val_groupmems_set}|g" $tmpDir/${bin_package}_useradd.xinst
		cat $tmpDir/${bin_package}_useradd.xinst >> ${tmpScript}
		
	fi

	# Generate the full script
	if [ -f ${tmpScript} ]; then
		install -m 777 $templateFile $script_file
		cat ${tmpScript} >> $script_file
	fi

	# Delete temporary workdir
	rm -rf $tmpDir
}

generate_postinst() {
	local bin_package="$1"
	local package="$2"
	local suffix=$(shell_variable_name $package)
	local script_file="$metadataTempDir/${bin_package}.postinst"
	local templatesDir="${script_lib_dir}/xinstall"
	local templateFile="${script_lib_dir}/xinstall/template.xinst"
	local tmpDir="$metadataTempDir/.ac6_${bin_package}"
	local tmpScript="$tmpDir/tmp_${bin_package}.preinst"
	
	# Create temporary workdir
	install -d $tmpDir
	
	# SystemD services
	spec_systd_services_var=SYSTEMD_SERVICES_${suffix}
	val_systd_services_set=${!spec_systd_services_var}
	if [ -n "$val_systd_services_set" ]; then
		install ${templatesDir}/systemd.xinst $tmpDir/${bin_package}_systemd.xinst
		sed -i -e "s|@SYSTEMD_SERVICES@|${val_systd_services_set}|g" $tmpDir/${bin_package}_systemd.xinst
		cat $tmpDir/${bin_package}_systemd.xinst >> ${tmpScript}
	fi

	# Generate the full script
	if [ -f ${tmpScript} ]; then
		install -m 777 $templateFile $script_file
		cat ${tmpScript} >> $script_file
	fi
	
	# Delete temporary workdir
	rm -rf $tmpDir
}
